import pydicom


class EquipmentAnonymizer:
    def __init__(self, address_anonymizer):
        self.address_anonymizer = address_anonymizer

        self._element_anonymizers = {
            pydicom.datadict.tag_for_keyword("InstitutionName"): self.anonymize_institution_name,
            pydicom.datadict.tag_for_keyword("InstitutionAddress"): self.anonymize_institution_address,
            pydicom.datadict.tag_for_keyword("InstitutionalDepartmentName"): self.anonymize_department_name,
            pydicom.datadict.tag_for_keyword("StationName"): self.anonymize_station_name,
        }

    def __call__(self, dataset, data_element):
        element_anonymizer = self._element_anonymizers.get(data_element.tag, None)
        if not element_anonymizer:
            return False

        element_anonymizer(dataset, data_element)

    def anonymize_institution_name(self, dataset, data_element):
        region = self.address_anonymizer.get_region(data_element.value)
        dataset.InstitutionAddress = " ".join(
            [
                self.address_anonymizer.get_street_address(data_element.value),
                region,
                self.address_anonymizer.get_country(data_element.value),
            ]
        )
        data_element.value = region + " CLINIC"

    def anonymize_institution_address(self, dataset, data_element):
        # handled by anonymize_institution_name
        pass

    def anonymize_department_name(self, dataset, data_element):
        data_element.value = "RADIOLOGY"

    def anonymize_station_name(self, dataset, data_element):
        data_element.value = dataset.Modality + "01"



import pydicom


class UnwantedElementsStripper:
    def __init__(self, *tag_names):
        self.tags = [pydicom.datadict.tag_for_keyword(tag_name) for tag_name in tag_names]

    def __call__(self, dataset, data_element):
        if data_element.tag in self.tags:
            del dataset[data_element.tag]
            return True
        return False



import collections
import datetime
import pydicom
import pydicom.dataelem
import random


class UIAnonymizer:
    def __init__(self):
        self._ui_map = collections.defaultdict(self._new_ui)

    def __call__(self, dataset, data_element):
        if (
            data_element.VR != "UI"
            or not data_element.value
            or pydicom.datadict.keyword_for_tag(data_element.tag).endswith("ClassUID")
            or data_element.tag == pydicom.datadict.tag_for_keyword("TransferSyntaxUID")
        ):
            return False

        if pydicom.dataelem.isMultiValue(data_element.value):
            data_element.value = list([self._ui_map[v] for v in data_element.value])
        else:
            data_element.value = self._ui_map[data_element.value]
        return True

    def _new_ui(self):
        date_part = datetime.datetime.utcnow().strftime("%Y%m%d%H%M%S")
        return "2." + date_part + "." + str(random.randint(1e45, 1e46 - 1))



import pydicom


class FixedValueAnonymizer:
    def __init__(self, tag_name, value):
        self.tag = pydicom.datadict.tag_for_keyword(tag_name)
        self.value = value

    def __call__(self, dataset, data_element):
        if data_element.tag == self.tag:
            data_element.value = self.value
            return True
        return False



import pydicom


class IDAnonymizer:
    _alphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"

    def __init__(self, randomizer, id_prefix, id_suffix, *tag_names):
        self.randomizer = randomizer
        self.id_prefix = id_prefix
        self.id_suffix = id_suffix
        self.issuer_tag = pydicom.datadict.tag_for_keyword("IssuerOfPatientID")
        self.id_tags = [pydicom.datadict.tag_for_keyword(tag_name) for tag_name in tag_names]

        total_affixes_length = len(self.id_prefix) + len(self.id_suffix)
        self._indices_for_randomizer = [len(self._alphabet)] * (12 - total_affixes_length)

    def __call__(self, dataset, data_element):
        if data_element.tag in self.id_tags:
            if isinstance(data_element.value, pydicom.multival.MultiValue):
                data_element.value = [self._new_id(id) for id in data_element.value]
            else:
                data_element.value = self._new_id(data_element.value)
            return True
        if data_element.tag == self.issuer_tag and data_element.value:
            data_element.value = "DICOGNITO"
        return False

    def _new_id(self, original_value):
        indexes = self.randomizer.get_ints_from_ranges(original_value, *self._indices_for_randomizer)
        id_root = "".join([self._alphabet[i] for i in indexes])
        return self.id_prefix + id_root + self.id_suffix



"""dicognito - anonymize DICOM files"""
from __future__ import print_function
import sys
import argparse
import collections
import glob
import os.path
import logging
import pydicom

from dicognito.anonymizer import Anonymizer


def main(args=None):
    if args is None:
        args = sys.argv[1:]

    parser = argparse.ArgumentParser(description="Anonymize one or more DICOM files.")
    parser.add_argument(
        "sources",
        metavar="source",
        type=str,
        nargs="+",
        help="The directories or file globs (e.g. *.dcm) to anonymize. Directories "
        "will be recursed, and all files found within will be anonymized.",
    )
    parser.add_argument(
        "--id-prefix",
        "-p",
        default="",
        help="A short string prepended to each ID field, such as PatientID, "
        "AccessionNumber, and so on, to make it easier to identify anonymized "
        "studies. Longer prefixes reduce the number of available random "
        "characters in the ID and increase the chance of collisions with other "
        "IDs. May be combined with --id-suffix.",
    )
    parser.add_argument(
        "--id-suffix",
        "-s",
        default="",
        help="A short string appended to each ID field, such as PatientID, "
        "AccessionNumber, and so on, to make it easier to identify anonymized "
        "studies. Longer suffixes reduce the number of available random "
        "characters in the ID and increase the chance of collisions with other "
        "IDs. May be combined with --id-prefix.",
    )
    parser.add_argument(
        "--quiet",
        "-q",
        action="store_true",
        help="Reduce the verbosity of output. Suppresses summary of anonymized studies.",
    )
    parser.add_argument(
        "--log-level",
        action="store",
        default="WARNING",
        help="Set the log level. May be one of DEBUG, INFO, WARNING, ERROR, or CRITICAL.",
    )
    parser.add_argument(
        "--salt",  # currently only intended to make testing easier
        help="The salt to use when generating random attribute values. Primarily "
        "intended to make testing easier. Best anonymization practice is to omit "
        "this value and let dicognito generate its own random salt.",
    )

    args = parser.parse_args(args)

    numeric_level = getattr(logging, args.log_level.upper(), None)
    if not isinstance(numeric_level, int):
        raise ValueError("Invalid log level: %s" % args.log_level)
    logging.basicConfig(format="", level=numeric_level)

    anonymizer = Anonymizer(id_prefix=args.id_prefix, id_suffix=args.id_suffix, salt=args.salt)

    ConvertedStudy = collections.namedtuple("ConvertedStudy", ["AccessionNumber", "PatientID", "PatientName"])

    def get_files_from_source(source):
        if os.path.isfile(source):
            yield source
        elif os.path.isdir(source):
            for (dirpath, dirnames, filenames) in os.walk(source):
                for filename in filenames:
                    yield os.path.join(dirpath, filename)
        else:
            for expanded_source in glob.glob(source):
                for file in get_files_from_source(expanded_source):
                    yield file

    converted_studies = set()
    for source in args.sources:
        for file in get_files_from_source(source):
            try:
                with pydicom.dcmread(file, force=False) as dataset:
                    anonymizer.anonymize(dataset)
                    dataset.save_as(file, write_like_original=False)
                    converted_studies.add(
                        ConvertedStudy(dataset.AccessionNumber, dataset.PatientID, str(dataset.PatientName))
                    )
            except pydicom.errors.InvalidDicomError:
                logging.info("File %s appears not to be DICOM. Skipping.", file)
                continue

    if not args.quiet:
        headers = ("Accession Number", "Patient ID", "Patient Name")
        print("%-16s %-16s %s" % headers)
        print("%-16s %-16s %s" % tuple("-" * len(header) for header in headers))
        for converted_study in sorted(converted_studies, key=lambda k: (k.PatientID, k.AccessionNumber)):
            print("%-16s %-16s %s" % converted_study)


if __name__ == "__main__":
    main()



import hashlib
import os


class Randomizer:
    def __init__(self, salt):
        if salt is None:
            salt = os.urandom(20)
        self.salt = str(salt)

    def to_int(self, original_value):
        message = self.salt + str(original_value)
        if isinstance(message, bytes):
            encoded = message
            hash = [ord(d) for d in hashlib.md5(encoded).digest()]
        else:
            encoded = message.encode("utf8")
            hash = hashlib.md5(encoded).digest()
        result = 0
        for c in hash:
            result *= 0x100
            result += c
        return result

    def get_ints_from_ranges(self, original_value, *suprenums):
        big_int = self.to_int(original_value)
        result = []
        for s in suprenums:
            result.append(big_int % s)
            big_int //= s
        return result



import datetime


class DateTimeAnonymizer:
    def __init__(self, offset_hours):
        self.offset = datetime.timedelta(hours=offset_hours)

    def __call__(self, dataset, data_element):
        if data_element.VR != "DA" and data_element.VR != "DT":
            return False
        if not data_element.value:
            return True

        if data_element.VR == "DA":
            self._anonymize_date_and_time(dataset, data_element)
        else:
            self._anonymize_datetime(dataset, data_element)
        return True

    def _anonymize_date_and_time(self, dataset, data_element):
        date_value = data_element.value
        date_format = "%Y%m%d"[: len(date_value) - 2]

        old_date = datetime.datetime.strptime(date_value, date_format).date()

        old_hours = datetime.time()
        time_value = ""
        time_name = data_element.keyword[:-4] + "Time"
        if time_name in dataset:
            time_value = dataset.data_element(time_name).value
            if time_value:
                old_hours = datetime.datetime.strptime(time_value[:2], "%H").time()

        old_datetime = datetime.datetime.combine(old_date, old_hours)
        new_datetime = old_datetime + self.offset

        data_element.value = new_datetime.strftime(date_format)
        if time_value:
            dataset.data_element(time_name).value = new_datetime.strftime("%H") + time_value[2:]

    def _anonymize_datetime(self, dataset, data_element):
        datetime_value = data_element.value
        datetime_format = "%Y%m%d%H"[: len(datetime_value) - 2]

        old_datetime = datetime.datetime.strptime(datetime_value[:10], datetime_format)
        new_datetime = old_datetime + self.offset

        new_datetime_value = new_datetime.strftime(datetime_format)
        new_datetime_value += datetime_value[len(new_datetime_value) :]
        data_element.value = new_datetime_value



import setuptools

package_name = "dicognito"
url = "https://github.com/blairconrad/dicognito"

with open("release_notes.md", "r") as notes:
    for line in notes:
        if line.startswith("## "):
            version = line[3:].strip()
            break

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name=package_name,
    version=version,
    author="Blair Conrad",
    author_email="blair@blairconrad.com",
    description="A tool for anonymizing DICOM files",
    license="MIT",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url=url,
    download_url="%(url)s/archive/%(version)s.tar.gz" % vars(),
    package_dir={"": "src"},
    packages=setuptools.find_packages("src"),
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Development Status :: 3 - Alpha",
        "Environment :: Console",
        "Intended Audience :: Healthcare Industry",
    ],
    install_requires=["pydicom"],
    entry_points={"console_scripts": ["dicognito=dicognito.__main__:main"]},
)



from dicognito.addressanonymizer import AddressAnonymizer
from dicognito.equipmentanonymizer import EquipmentAnonymizer
from dicognito.fixedvalueanonymizer import FixedValueAnonymizer
from dicognito.idanonymizer import IDAnonymizer
from dicognito.pnanonymizer import PNAnonymizer
from dicognito.datetimeanonymizer import DateTimeAnonymizer
from dicognito.uianonymizer import UIAnonymizer
from dicognito.unwantedelements import UnwantedElementsStripper
from dicognito.randomizer import Randomizer

import random


class Anonymizer:
    def __init__(self, id_prefix="", id_suffix="", salt=None):
        minimum_offset_hours = 62 * 24
        maximum_offset_hours = 730 * 24
        randomizer = Randomizer(salt)
        address_anonymizer = AddressAnonymizer(randomizer)
        self._element_handlers = [
            UnwantedElementsStripper(
                "BranchOfService",
                "Occupation",
                "MedicalRecordLocator",
                "MilitaryRank",
                "PatientInsurancePlanCodeSequence",
                "PatientReligiousPreference",
                "PatientTelecomInformation",
                "PatientTelephoneNumbers",
                "ReferencedPatientPhotoSequence",
                "ResponsibleOrganization",
            ),
            UIAnonymizer(),
            PNAnonymizer(randomizer),
            IDAnonymizer(
                randomizer,
                id_prefix,
                id_suffix,
                "AccessionNumber",
                "OtherPatientIDs",
                "PatientID",
                "PerformedProcedureStepID",
                "RequestedProcedureID",
                "ScheduledProcedureStepID",
                "StudyID",
            ),
            address_anonymizer,
            EquipmentAnonymizer(address_anonymizer),
            FixedValueAnonymizer("RequestingService", ""),
            FixedValueAnonymizer("CurrentPatientLocation", ""),
            DateTimeAnonymizer(-random.randint(minimum_offset_hours, maximum_offset_hours)),
        ]

    def anonymize(self, dataset):
        dataset.file_meta.walk(self._anonymize_element)
        dataset.walk(self._anonymize_element)

    def _anonymize_element(self, dataset, data_element):
        for handler in self._element_handlers:
            if handler(dataset, data_element):
                return
